# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-22 05:56:32 +0900 (Wed, 22 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/file'
require 'vfs/scheme/file/common'

module VFS;  module SchemeHandler; module FILE;
  class File < VFS::File
    include VFS::SchemeHandler::FILE::Common
    extend  VFS::SchemeHandler::FILE::MethodGenerator

    relay_realpath_to_file_builtin :size?,
      :chardev?, :zero?, :writable_real?,
      :atime, :pipe?, :exists?, :symlink, :sticky?, :file?, :stat,
      :blockdev?, :writable?, :ftype, :foreach,
      :grpowned?, :exist?, :link, :setgid?, :executable_real?,
      :delete, :ctime, :socket?, :readable_real?, :truncate, :unlink,
      :lstat, :owned?, :directory?, :setuid?, :executable?,
      :expand_path, :mtime, :symlink?, :size, :readable?, :readlink

    undef :buf, :open_for_read, :open_for_write

    def open(*arg, &block)
      FileBuiltIn.open(uri.path, *arg, &block)
    end

    def rename_internal(dest)
      begin
        FileBuiltIn.rename(uri.path, dest)
        VFS::File.new(dest)
      rescue
        rename_external(dest)
      end
    end

    def utime(n_atime, n_mtime)
      FileBuiltIn.utime(n_atime, n_mtime, uri.path)
      self
    end

    def chmod(mode)
      FileBuiltIn.chmod(mode, uri.path)
      self
    end

    def lchmod(mode)
      FileBuiltIn.lchmod(mode, uri.path)
      self
    end

    def chown(user, group)
      FileBuiltIn.chown(user, group, uri.path)
      self
    end

    def lchown(user, group)
      FileBuiltIn.lchown(user, group, uri.path)
      self
    end

    class_eval [:uid, :dev, :rdev_major, :nlink, :blocks, :rdev,
                :mode, :blksize, :gid, :ino, :rdev_minor].map {|m| %{
        def #{m}(*args)
          FileBuiltIn::Stat.new(uri.path).#{m}(*args)
        end
      } }.join

  end # VFS::SchemeHandler::FILE::File
end; end; end

if __FILE__ == $0
  uri = URI.parse("file:///home/sugi/.bashrc")
  vfs = VFS::SchemeHandler::FILE::File.new(uri)
  p vfs
  p vfs.join(%w(a b c))
  p VFS::join(%w(a b c))
  p VFS::scheme(uri)
  p vfs.scheme
  p vfs.mtime
#  p VFS::SchemeHandler::FILE::File.mtime(f)
  p vfs.basename
  p vfs.dirname
  p vfs.scheme
  io = vfs.open
  puts io.read(20)
  io.close
  vfs.open { |io| puts io.read(20) }
#  io = VFS::SchemeHandler::FILE::File.open(f)
#  puts io.read(20)
#  io.close
#  VFS::SchemeHandler::FILE::File.open(f){ |io| puts io.read(20) }
#  VFS::SchemeHandler::FILE::File.new("/tmp/moge") << ["mogemoge", Time.new]
end
