# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-11-10 23:43:40 +0900 (Mon, 10 Nov 2003) $
#   $LastChangedBy: sugi $
#

require 'vfs/file'
require 'vfs/scheme/http/common'

module VFS;  module SchemeHandler;
  module HTTP;
    class File < VFS::File
      include VFS::SchemeHandler::HTTP::Common
      # FIXME, TODO :utime, :rename, :truncate,
      # :lstat :stat :chown(hmm?)
      # :umask(hmm?), :expand_path :readlink

      return_false :chardev?, :pipe?, :sticky?, :blockdev?, :grpowned?,
        :setgid?, :executable_real?, :socket?, :owned?, :setuid?,
        :executable?, :symlink?

      def initialize(uri)
	super
	check_scheme(@uri)
	init_common_attr
	@mtime          = nil
	@ctime          = nil
	@size           = nil
	@is_exists      = nil
        @is_fetched     = false
      end
      attr_writer :size, :mtime
      private :size=, :mtime=

      def fetched?
        @is_fetched
      end

      def readable?
	# fixme? should not catch exception? check.
	begin
	  header
	rescue
	  return false
	end
	true
      end
      alias :readable_real? :readable?

      def writable?
	if directory?
	  capability?("MKCOL") || capability?("DELETE")
	else
	  capability? "PUT"
	end
      end
      alias :writable_real? :writable? 

      def file?
	exists? && ! directory?
      end

      def directory?
        begin
          property("D:getcontenttype") =~ /httpd\/unix-directory/i ? true : false
	rescue Net::ProtoFatalError # only for 1.6
          false
        rescue Net::HTTPServerException # FIXME. original error class or ERRNO
          false
        end
      end

      def size
	@size = property("lp\d:getcontentlength") ||
	  header("content-length")
	if @size != nil
	  @size = @size.to_i
	else
	  wait_fetch
	end
	@size
      end

      def mtime
        require 'time.rb'
        @mtime = property("lp\d:getlastmodified") ||
                 header("last-modified")
	if @mtime != nil
	  @mtime = Time.parse(@mtime)
	end
	@mtime
      end
      # fixme; atime?
      alias :atime :mtime

      def ctime
        require 'time.rb'
	if property("lp\d:getlastmodified") != nil
	  @ctime = Time.at(property("lp\d:getlastmodified"))
	end
	@ctime
      end

      def exists?
	return @is_exists unless @is_exists == nil
	begin
	  header
	rescue Net::ProtoFatalError # only for 1.6
	  return @is_exists = false
        rescue Net::HTTPServerException # FIXME. original error class or ERRNO
	  return @is_exists = false
	end
	@is_exists = true
      end
      alias :exist? :exists?

      def wait_fetch
        fetch unless fetched?
      end

      def delete
	# FIXME; correct error handling
	http_request(:delete)
      end
      alias :unlink :delete

      private

      def fetch
	log("Start fetching.")
        http_request(:get) { |rep|
          buf << rep.body
        }
        buf.close
        if @size == nil
          @size = buf.size
        end
        log("HTTP fetching done.")
        @is_fetched = true
      end

      def rename_internal(dest)
	if uri.host != dest.host
	  rename_external(dest)
	else
	  begin
	    # FIXME; correct error handling
	    http_request(:move,	{'Overwrite' => 'T',
			         'Destination' => dest.to_s})
	    @uri = dest
	    self
#	  rescue HTTPResponse # Hmmm? OK?
#	    rename_external(dest)
	  end
	end # if uri.host != dest.host
      end

      def open_for_read(flag = RDONLY, perm = nil, &block)
	wait_fetch
	super
      end

      def open_for_write(flag = WRONLY, perm = nil, &block)
	flag = conv_openflang_to_const(flag)
	if flag & CREAT == 0
	  log("no create moge. checking existance.", Log::DEBUG)
	  # FIXME. change exception class.
	  exists? or raise Errno::ENOENT
	end
	if flag & RDWR != 0 && flag & TRUNC == 0
	  log("read-write moge. waiting fetch.", Log::DEBUG)
          exists? && wait_fetch
	end
	if flag & APPEND != 0
	  log("Append moge. set pointer to end of TempBuf.", Log::DEBUG)
	  buf.seek(buf.size)
	end

	buf.writeback_proc = Proc.new {
	  # for writeback to server
	  put_header = { "Content-Type" => "application/octat-stream" }
	  buf.seek(0)
	  if flag & APPEND != 0 && exists?
            cur_size   = size?.to_i
	    put_header["Content-Range"] = "bytes #{cur_size.to_s}-" +
	      (cur_size + buf.size - 1).to_s + "/" + 
	      (cur_size + buf.size).to_s
	    log("Append moge. content-range: #{put_header["Content-Range"]}",
		Log::INFO)
	  end
	  # FIXME. must catch error
	  http_request(:put, buf, put_header ) { |rep|
	    log("PUT success. #{rep.inspect}")
	  }
	} # end of close hook for writeback to server

	if block_given?
	  begin
	    yield(buf)
	  ensure
	    buf.close
	  end
	else
	  buf
	end
      end # open_for_write

    end # VFS::SchemeHandler::Http::File
end; end; end


if __FILE__ == $0
#  uri = "http://localhost/"
#  uri = "http://localhost/~sugi/"
  uri = "http://sugi.nemui.org/pub/prog/hns2td"
  f = VFS::SchemeHandler::HTTP::File.new(uri)
  p f
  p VFS::scheme(uri)
  p f.scheme
  p f.exists?
  p f.directory?
  p f.writable?
  p f.mtime
#  p VFS::SchemeHandler::HTTP::File.mtime(f) # something wrong; warning: method redefined
  p f.size
#  p VFS::SchemeHandler::HTTP::File.size(f)
  p f.basename
  p f.dirname
  io = f.open
  puts io.read(20)
  io.close
  f.open { |io| puts io.read(20) }
#  io = VFS::SchemeHandler::HTTP::File.open(f)
#  puts io.read(20)
#  io.close
#  VFS::SchemeHandler::HTTP::File.open(f){ |io| puts io.read(20) }

  uri = "http://sugi.nemui.org/pub/incomming/writetest"
  f = VFS::SchemeHandler::HTTP::File.new(uri)
  p f.exists?
  f.open("a") { |io|
    io << VFS::File.open("/etc/motd").read
  }
  puts f.open.read
  f.rename("http://sugi.nemui.org/pub/incomming/movetest")
  p f
  f.delete
end
